package com.ruoyi.utils.resumeAnalysis;


import com.ruoyi.system.domain.resume.Resume;
import com.ruoyi.utils.resumeAnalysis.similarity.Similarity;
import com.hankcs.hanlp.HanLP;
import com.hankcs.hanlp.corpus.tag.Nature;
import com.hankcs.hanlp.dictionary.CustomDictionary;
import com.hankcs.hanlp.seg.Segment;
import com.hankcs.hanlp.seg.common.Term;
import com.hankcs.hanlp.tokenizer.NLPTokenizer;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.stream.Collectors;


public class HanLPUtils {

    /**
     * 分词词典工具类
     */
    private static CustomDictionary customDictionary = new CustomDictionary();

    /***
     * hanlp分词
     * @param input
     */
    public static Map<String, String> getOut(String input){
        Map<String, String> resMap = new HashMap<>();

        Segment segment =  HanLP.newSegment();  //使用Hanlp分词
        List<Term> termList = segment.seg(input);

        //遍历分词结果
        for (Term term : termList) {
            String word = term.toString().substring(0, term.length());      //词
            String nature = term.toString().substring(term.length() + 1);   //词性
            if (StringUtils.isNotBlank(word) && StringUtils.isNotBlank(nature)) {
                //将词及词性放到Map结果集中
                resMap.put(word, nature);
            }
        }
        return resMap;
    }

    /**
     * 获取关联人姓名
     *
     * @param nameString 姓名字符串
     * @return 关联人姓名
     */
    private static List<String> getRelationName(String nameString) {
        List<String> standard = new ArrayList<>();
        List<String> tokenizer = new ArrayList<>();
        try {
            standard = standardParticiple(nameString);
            tokenizer = nlpTokenizer(nameString);
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        standard.addAll(tokenizer);
        return standard.stream().distinct().collect(Collectors.toList());
    }


    /**
     * 标准分词
     *
     * @param nameString 要分析的字符串
     * @return 人名
     */
    private static List<String> standardParticiple(String nameString) {
        List<Term> termList = HanLP.segment(nameString);
        //System.out.println(termList);
        List<Term> collect = termList.stream().filter(x -> x.nature.equals(Nature.nr)).collect(Collectors.toList());
        return collect.stream().map(term -> term.word).collect(Collectors.toList());
    }

    /**
     * NLP分词
     *
     * @param nameString 要分析的字符串
     * @return 人名
     */
    private static List<String> nlpTokenizer(String nameString) {
        NLPTokenizer.ANALYZER.enableCustomDictionary(false);
        List<Term> termList = NLPTokenizer.segment(nameString);
        //System.out.println(termList);
        List<Term> collect = termList.stream().filter(x -> x.nature.equals(Nature.nr)).collect(Collectors.toList());
        return collect.stream().map(term -> term.word).collect(Collectors.toList());
    }

    public static boolean isIDNumber(String IDNumber) {
        if (IDNumber == null || "".equals(IDNumber)) {
            return false;
        }
        // 定义判别用户身份证号的正则表达式（15位或者18位，最后一位可以为字母）
        String regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" +
                "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";
        //假设18位身份证号码:41000119910101123X  410001 19910101 123X
        //^开头
        //[1-9] 第一位1-9中的一个      4
        //\\d{5} 五位数字           10001（前六位省市县地区）
        //(18|19|20)                19（现阶段可能取值范围18xx-20xx年）
        //\\d{2}                    91（年份）
        //((0[1-9])|(10|11|12))     01（月份）
        //(([0-2][1-9])|10|20|30|31)01（日期）
        //\\d{3} 三位数字            123（第十七位奇数代表男，偶数代表女）
        //[0-9Xx] 0123456789Xx其中的一个 X（第十八位为校验值）
        //$结尾

        //假设15位身份证号码:410001910101123  410001 910101 123
        //^开头
        //[1-9] 第一位1-9中的一个      4
        //\\d{5} 五位数字           10001（前六位省市县地区）
        //\\d{2}                    91（年份）
        //((0[1-9])|(10|11|12))     01（月份）
        //(([0-2][1-9])|10|20|30|31)01（日期）
        //\\d{3} 三位数字            123（第十五位奇数代表男，偶数代表女），15位身份证不含X
        //$结尾


        boolean matches = IDNumber.matches(regularExpression);

        //判断第18位校验值
        if (matches) {

            if (IDNumber.length() == 18) {
                try {
                    char[] charArray = IDNumber.toCharArray();
                    //前十七位加权因子
                    int[] idCardWi = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
                    //这是除以11后，可能产生的11位余数对应的验证码
                    String[] idCardY = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
                    int sum = 0;
                    for (int i = 0; i < idCardWi.length; i++) {
                        int current = Integer.parseInt(String.valueOf(charArray[i]));
                        int count = current * idCardWi[i];
                        sum += count;
                    }
                    char idCardLast = charArray[17];
                    int idCardMod = sum % 11;
                    if (idCardY[idCardMod].toUpperCase().equals(String.valueOf(idCardLast).toUpperCase())) {
                        return true;
                    } else {
                        return false;
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                    return false;
                }
            }

        }
        return matches;
    }



    public static void main(String[] main) throws Exception {

        /**
         * 定义参数（可通过request传输参数进行修改）
         */
        //文件路径
//        String filePath = "C:/Users/Administrator/Desktop/张继超/DGT-京东-前端-张海霞-北京.pdf";
        //OCR识别文字输出路径
//        String txtFilePath = "C:/Users/Administrator/Desktop/张继超/DGT-京东-前端-张海霞-北京.txt";

        //pdf转图片
//        List<String> pngList = PdfFileUtil.pdf2png(filePath,"PNG");
//        //循环所有图片进行文字识别
//        for (String imgFile: pngList) {
//            AliCloudOCRUtils.getImgToTxtAnalysis(txtFilePath,imgFile);
//        }

//        String file_key = "f43d9760812a32dc9d9ee674b6a9cdc0";//1667441627780.doc
        //1667957783110
//        String Path = "C:\\Users\\Administrator\\Desktop\\张继超\\1669778120747.doc";

//		byte[] data = FileUtils.getImageStream(Path);
//		Resume resume = switchParseTools(data,"2",Path);


//        HanLPUtils.analyzeResume(result);

//        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
//        String ymd = sdf.format(new Date());
//        String lastPath = "/" + ymd + "/";
//        //建立文件夹
//        File f = new File(Config.getProperty("attachment_save_path") + lastPath);
//        if (!f.exists()) {
//            f.mkdirs();
//        }

//        String filePath = "\\home\\quizgo\\pdf2word\\";
//        String Path = AliPDFToWordUtils.getPDFToWord(filePath,"\\home\\quizgo\\pdf2word\\");

//        String Path = "C:\\Users\\Administrator\\Desktop\\张继超\\1669798761043.doc";
////        String Path = "C:\\Users\\Administrator\\Desktop\\张继超\\1667438649530s.doc";
//        //读取word内容
//        List<String> result = FileUtils.readWord(Path);
//
//        //获取标识和基础信息相似度
//        Map<String, Map<String,Object>> resultMap = getResumeSimilarity(result);
//        System.out.println(resultMap);
//        //处理标识分段，并判断内容情况
//        Resume resume = similaritySubsection(resultMap,result);
//        System.out.println(resume);

        String str = "目前的功能主要有捐赠人管理用于添加、查看捐赠人信息。捐赠人台账用于管理捐赠人捐赠款项的匹配与发布。捐赠人虚拟账户用于管理捐赠人捐赠款项分配。捐赠人银行账户管理捐赠人银行账户。捐赠人对接人用于对对接人的管理包括添加、停用、重置密码、发送邮件。";
        boolean isTitle = ResumeAnalysisUtil.isTitle(str);


//        String filePath = "C:/Users/Administrator/Desktop/张继超/张继超-个人简历.pdf";
//        //OCR识别文字输出路径
//        String txtFilePath = "C:/Users/Administrator/Desktop/张继超/张继超-个人简历.txt";
//
//        //OCR识别文字输出路径
////        String txtFilePath = "C:/Users/Administrator/Desktop/张继超/DGT-京东-前端-张海霞-北京.txt";
//        Map<String, Object> resumeMap = HanLPUtils.analyzeResume(txtFilePath);
//        System.out.println(resumeMap);

    }


    public static Resume similaritySubsection(Map<String, Map<String,Object>> resultMap, List<String> result){

        Resume resume = new Resume();
        /**
         * 定义分段的数据结构
         */
        List<String> list = new ArrayList<>();
        for (Map.Entry<String, Map<String,Object>> entry : resultMap.entrySet()) {
            Map<String,Object> map = entry.getValue();
            if((boolean)map.get("isTitle")){
                list.add(map.get("coordinate").toString());
            }
        }
        Collections.sort(list);

        /**
         * 处理数据
         */
        List<Map<String,String>> paragraphList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            String coordinate = list.get(i);
            Map<String,String> paragraph = new HashMap<>();
            for (Map.Entry<String, Map<String,Object>> entry : resultMap.entrySet()) {
                Map<String,Object> map = entry.getValue();
                if((boolean)map.get("isTitle")){
                    if(Integer.valueOf(coordinate) == (Integer) map.get("coordinate")){
                        paragraph.put("title",entry.getKey());
                    }
                }
                if(!(boolean)map.get("isTitle")){
                    if(i+1 < list.size() &&
                            Integer.valueOf(coordinate)<(Integer) map.get("coordinate")&&
                            Integer.valueOf(list.get(i+1))>(Integer) map.get("coordinate")){
                        //获取段落
                        paragraph.put(map.get("coordinate").toString(),entry.getKey());
                    } else if (i+1 == list.size()
                            &&Integer.valueOf(coordinate)<(Integer) map.get("coordinate")) {
                        paragraph.put(map.get("coordinate").toString(),entry.getKey());
                    }
                }
            }
            paragraphList.add(paragraph);
        }

        Map<String,Object> maps = getSimilarityData(resultMap);
        /**
         * 分段数据结束展示
         */
        //System.out.println(paragraphList);
        /**
         * 处理分段数据拆分
         */
        for (Map<String,String> str:paragraphList) {
            String title = str.get("title");
            if(StringUtils.isNotEmpty(title)){

                //map排序
                Map<String,String> mapByKey = sortMapByKey(str);
                if("工作经历".equals(StringFilter(title))){

                    List<String> companyList = new ArrayList<>();
                    List<String> dateList = new ArrayList<>();

                    for (Map.Entry<String, String> entry : mapByKey.entrySet()) {
                        if(!entry.getKey().equals("title")){
                            //读取单位名称
                            String companyName = getCompanyName(entry.getValue());
                            if(StringUtils.isNotEmpty(companyName)){
                                companyList.add(companyName);
                            }
                            //读取职位
                        }
                    }
                }
                if("项目经验".equals(StringFilter(title))){
                    String keyWords = "";
                    for (Map.Entry<String, String> entry : mapByKey.entrySet()) {
                        if(!entry.getKey().equals("title")){
                            keyWords += entry.getValue();
                        }
                    }
                    resume.setWorkExperience(keyWords);
                }
                if("掌握技能".equals(StringFilter(title))){
                    String keyWords = "";
                    for (Map.Entry<String, String> entry : mapByKey.entrySet()) {
                        if(!entry.getKey().equals("title")){
                            keyWords += entry.getValue()+"\r\n";
                        }
                    }
                    resume.setKeyWords(keyWords);
                }
                if("求职意向".equals(StringFilter(title))){
                    String jobObjective = "";
                    for (Map.Entry<String, String> entry : mapByKey.entrySet()) {
                        if(!entry.getKey().equals("title")){
                            jobObjective += entry.getValue();
                        }
                    }
                    resume.setJobObjective(jobObjective);
                }
                if("自我介绍".equals(StringFilter(title))){
                    String selfIntroduce = "";
                    for (Map.Entry<String, String> entry : mapByKey.entrySet()) {
                        if(!entry.getKey().equals("title")){
                            selfIntroduce += entry.getValue();
                        }
                    }
                    resume.setSelfIntroduce(selfIntroduce);
                }
            }
        }

        /**
         * 基础信息的相似度对比获取
         */
//        System.out.println(maps);

        //姓名提取判断
        if(StringUtils.isNotEmpty(maps.get("regularFullName").toString())){
            String regularFullName = maps.get("regularFullName").toString();
            if(StringUtils.isNotEmpty(regularFullName)){
                resume.setUserName(StringFilter(regularFullName).replaceAll("姓名",""));
            }
        } else if (StringUtils.isNotEmpty(maps.get("relationName").toString())) {
            String relationName = maps.get("relationName").toString();
            if(StringUtils.isNotEmpty(relationName)){
                resume.setUserName(StringFilter(relationName).replaceAll("姓名",""));
            }
        }
        //性别
        if(StringUtils.isNotEmpty(maps.get("regularSex").toString())){
            String regularSex = maps.get("regularSex").toString();
            resume.setGender(StringFilter(regularSex).replaceAll("性别",""));
        } else if (StringUtils.isNotEmpty(maps.get("hanLPSex").toString())) {
            String hanLPSex = maps.get("hanLPSex").toString();
            resume.setGender(StringFilter(hanLPSex).replaceAll("性别",""));
        }
        //手机号码
        if(StringUtils.isNotEmpty(maps.get("regularTelephone").toString())){
            String regularTelephone = maps.get("regularTelephone").toString();
            resume.setMobile(StringFilter(regularTelephone));
        } else if (StringUtils.isNotEmpty(maps.get("hanLPTelephone").toString())) {
            String hanLPTelephone = maps.get("hanLPTelephone").toString();
            resume.setMobile(StringFilter(hanLPTelephone));
        }
        //邮箱
        if(StringUtils.isNotEmpty(maps.get("regularEmail").toString())){
            String regularEmail = maps.get("regularEmail").toString();
            resume.setEmail(regularEmail);
        } else if (StringUtils.isNotEmpty(maps.get("hanLPEmail").toString())) {
            String hanLPEmail = maps.get("hanLPEmail").toString();
            resume.setEmail(hanLPEmail.replaceAll("邮箱",""));
        }
        //身份证号
        if(StringUtils.isNotEmpty(maps.get("regularIdCard").toString())){
            String regularIdCard = maps.get("regularIdCard").toString();
            resume.setCardNo(regularIdCard);
        } else if (StringUtils.isNotEmpty(maps.get("hanLPIdCard").toString())) {
            String hanLPIdCard = maps.get("hanLPIdCard").toString();
            resume.setCardNo(hanLPIdCard);
        }
        //生日
        if(StringUtils.isNotEmpty(maps.get("regularBirthday").toString())){
            String regularBirthday = maps.get("regularBirthday").toString();
            resume.setAge(regularBirthday);
        } else if (StringUtils.isNotEmpty(maps.get("hanLPIdCard").toString())) {
            String hanLPIdCard = maps.get("hanLPIdCard").toString();
            resume.setAge(hanLPIdCard);
        }
        //学历获取
        if(StringUtils.isNotEmpty(maps.get("hanLPEducation").toString())){
            String hanLPEducation = maps.get("hanLPEducation").toString();
            resume.setDegree(hanLPEducation);
        }
        //学校
        if(null != maps.get("hanLPSchool")){
            String hanLPSchool = maps.get("hanLPSchool").toString();
            resume.setGraduateCollege(hanLPSchool);
        }
        //工作经验
        if(null!=maps.get("hanLPWorkExperience")){
            String hanLPWorkExperience = maps.get("hanLPWorkExperience").toString();
            resume.setExperience(hanLPWorkExperience);
        }
        //民族
        if(null !=  maps.get("hanLPNation")){
            String hanLPNation = maps.get("hanLPNation").toString();
            resume.setNationality(StringFilter(hanLPNation).replaceAll("民族",""));
        }

        return resume;
    }

    public static Map<String,Object> getSimilarityData(Map<String, Map<String,Object>> resultMap){

        Map<String,Object> personalMap = new HashMap<>();

        double isFullName = 0;
        String isFullNameCoordinate = "";
        String relationName = "";
        String regularFullName = "";

        double isSex = 0;
        String hanLPSex = "";
        String regularSex = "";

        double isTelephone = 0;
        String hanLPTelephone = "";
        String regularTelephone = "";

        double isEmail = 0;
        String hanLPEmail = "";
        String regularEmail = "";

        double isIdCard  = 0;
        String hanLPIdCard = "";
        String regularIdCard = "";

        double isBirthday  = 0;
        String hanLPBirthday = "";
        String regularBirthday = "";

        double isEducation = 0;
        String hanLPEducation = "";

        double isWorkExperience = 0;
        String hanLPWorkExperience = "";

        String hanLPNation = "";

        for (Map.Entry<String, Map<String,Object>> entry : resultMap.entrySet()) {
            Map<String,Object> map = entry.getValue();
            if(!(boolean)map.get("isTitle")){
                map.get("coordinate").toString();

                /**
                 * 姓名
                 * relationName
                 * isFullName
                 * regularFullName
                 */
                //存在相似度时进行相似度对比
                if(isFullName<(double) map.get("isFullName")){
                    isFullName = (double) map.get("isFullName");
                    isFullNameCoordinate = map.get("coordinate").toString();
                    if(StringUtils.isEmpty(relationName)){
                        relationName = map.get("relationName").toString();
                    }
                    regularFullName = map.get("regularFullName").toString();
                }else if((double) map.get("isFullName") == 0 && StringUtils.isNotEmpty(map.get("relationName").toString())){
                    //如果相似度不存在
                    //进行获取分词是否存在
                    if(StringUtils.isEmpty(relationName)){
                        relationName = map.get("relationName").toString();
                    }
                }

                /**
                 * 性别
                 */
                //存在相似度时进行相似度对比
                if(isSex<(double) map.get("isSex")){
                    isSex = (double) map.get("isSex");
                    hanLPSex = map.get("hanLPSex").toString();
                    if(StringUtils.isEmpty(regularSex)){
                        regularSex = map.get("regularSex").toString();
                    }
                }else if((double) map.get("isSex") == 0 && StringUtils.isNotEmpty(map.get("regularSex").toString())){
                    //如果相似度不存在
                    //进行获取分词是否存在
                    if(StringUtils.isEmpty(regularSex)){
                        regularSex = map.get("regularSex").toString();
                    }
                }

                /**
                 * 电话提取
                 */
                if(isTelephone<(double) map.get("isTelephone")){
                    isTelephone = (double) map.get("isTelephone");
                    hanLPTelephone = map.get("hanLPTelephone").toString();
                    if(StringUtils.isEmpty(regularTelephone)){
                        regularTelephone = map.get("regularTelephone").toString();
                    }
                }else if((double) map.get("isTelephone") == 0 && StringUtils.isNotEmpty(map.get("regularTelephone").toString())){
                    //如果相似度不存在
                    //进行获取分词是否存在
                    if(StringUtils.isEmpty(regularTelephone)){
                        regularTelephone = map.get("regularTelephone").toString();
                    }
                }

                /**
                 * 邮箱提取
                 */
                if(isEmail<(double) map.get("isEmail")){
                    isEmail = (double) map.get("isEmail");
                    hanLPEmail = map.get("hanLPEmail").toString();
                    if(StringUtils.isEmpty(regularEmail)){
                        regularEmail = map.get("regularEmail").toString();
                    }
                }else if(StringUtils.isNotEmpty(map.get("regularEmail").toString())){
                    //如果相似度不存在
                    //进行获取分词是否存在
                    if(StringUtils.isEmpty(regularEmail)){
                        regularEmail = map.get("regularEmail").toString();
                    }
                }

                /**
                 * 身份证提取
                 */
                if(isIdCard<(double) map.get("isIdCard")){
                    isIdCard = (double) map.get("isIdCard");
                    hanLPIdCard = map.get("hanLPIdCard").toString();
                    if(StringUtils.isEmpty(regularIdCard)){
                        regularIdCard = map.get("regularIdCard").toString();
                    }
                }else if(StringUtils.isNotEmpty(map.get("regularIdCard").toString())){
                    //如果相似度不存在
                    //进行获取分词是否存在
                    if(StringUtils.isEmpty(regularIdCard)){
                        regularIdCard = map.get("regularIdCard").toString();
                    }
                }

                /**
                 * 提取年龄
                 */
                if(0 < (double) map.get("isBirthday")){
                    if(isBirthday < (double) map.get("isBirthday")){
                        isBirthday = (double) map.get("isBirthday");
                    }
                    if(StringUtils.isEmpty(hanLPBirthday)){
                        hanLPBirthday = map.get("hanLPBirthday").toString();
                    }
                    if(StringUtils.isEmpty(regularBirthday)){
                        regularBirthday = map.get("regularBirthday").toString();
                    }
                }

                /**
                 *  学历获取
                 */
                if(1 == (double) map.get("isEducation")){
                    isEducation = (double) map.get("isEducation");
                    if(StringUtils.isEmpty(hanLPEducation)){
                        hanLPEducation = map.get("hanLPEducation").toString();
                    }
                }

                /**
                 * 学校获取
                 */
                if(StringUtils.isNotEmpty(map.get("hanLPSchool").toString())){
                    personalMap.put("hanLPSchool",map.get("hanLPSchool").toString());
                }

                /**
                 * 工作经验获取
                 */
                if(isWorkExperience < (double) map.get("isWorkExperience")){
                    isWorkExperience = (double) map.get("isWorkExperience");
                    personalMap.put("isWorkExperience",isWorkExperience);
                    if(StringUtils.isEmpty(hanLPWorkExperience)){
                        personalMap.put("hanLPWorkExperience",map.get("hanLPWorkExperience").toString());
                    }
                }
                //民资
                if(StringUtils.isNotEmpty(StringFilter(map.get("hanLPNation").toString()))){
                    hanLPNation = map.get("hanLPNation").toString();
                }

            }
        }

        personalMap.put("isFullName",isFullName);
        personalMap.put("isFullNameCoordinate",isFullNameCoordinate);
        personalMap.put("relationName",relationName);
        personalMap.put("regularFullName",regularFullName);

        personalMap.put("isSex",isSex);
        personalMap.put("hanLPSex",hanLPSex);
        personalMap.put("regularSex",regularSex);

        personalMap.put("isTelephone",isTelephone);
        personalMap.put("hanLPTelephone",hanLPTelephone);
        personalMap.put("regularTelephone",regularTelephone);

        personalMap.put("isEmail",isEmail);
        personalMap.put("hanLPEmail",hanLPEmail);
        personalMap.put("regularEmail",regularEmail);

        personalMap.put("isIdCard",isIdCard);
        personalMap.put("hanLPIdCard",hanLPIdCard);
        personalMap.put("regularIdCard",regularIdCard);

        personalMap.put("isBirthday",isBirthday);
        personalMap.put("hanLPBirthday",hanLPBirthday);
        personalMap.put("regularBirthday",regularBirthday);

        personalMap.put("isEducation",isEducation);
        personalMap.put("hanLPEducation",hanLPEducation);
        personalMap.put("hanLPNation",hanLPNation);


        return personalMap;
    }

    public static Map<String, Map<String,Object>> getResumeSimilarity(List<String> docContent){

        Map<String, Map<String,Object>> resultMap = new HashMap<>();

        for (int c = 0; c < docContent.size(); c++) {

            Map<String, Object> result = new HashMap<String, Object>();

            String txtContent = docContent.get(c);

            if(StringUtils.isNotEmpty(txtContent)){
                //内容下标
                result.put("coordinate",c);
                /**
                 * 先进行主要标识对比
                 */
                boolean isTitle = isTitle(txtContent);
                result.put("isTitle",isTitle);
                /**
                 * 如果是非特殊标识
                 * 进行其他标识相似度计算
                 */
                if(!isTitle){
                    /**
                     * 姓名相似度对比
                     */
                    //1、根据分词获取姓名
                    result.put("relationName",getRelationName(StringFilter(txtContent)));
                    //2、根据相似(姓名、名称、名字等)设置参数姓名行
                    result.put("isFullName",isFullName(StringFilter(txtContent)));
                    //3、根据正则表达式提取名字
                    result.put("regularFullName",regularFullName(StringFilter(txtContent)));

                    /**
                     * 性别相似度对比
                     */
                    //1、性别分词获取性别
                    result.put("hanLPSex",getHanLPSex(txtContent));
                    //2、根据相似度(男、女、性别)获取参数
                    result.put("isSex",isSimilarity(Arrays.asList("男","女","性别"),txtContent));
                    //3、根据正则表达式提取性别
                    result.put("regularSex",regularAcquisition("(男|女)",txtContent));

                    /**
                     * 读取手机号
                     */
                    //1、性别分词获取手机号
                    result.put("hanLPTelephone",getHanLPTelephone(txtContent));
                    //2、根据相似度手机号获取参数
                    result.put("isTelephone",isSimilarity(Arrays.asList("手机号码","电话"),txtContent));
                    //3、根据正则表达式提取手机号
                    result.put("regularTelephone",regularAcquisition("1[3-9]\\d{9}",txtContent));

                    /**
                     * 读取邮箱
                     */
                    //1、性别分词获取邮箱
                    result.put("hanLPEmail",getHanLEmail(txtContent));
                    //2、根据相似度邮箱获取参数
                    result.put("isEmail",isSimilarity(Arrays.asList("邮箱","email"),txtContent));
                    //3、根据正则表达式提取邮箱
                    result.put("regularEmail",regularAcquisition("^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$",txtContent));

                    /**
                     * 读取身份证号
                     */
                    //1、性别分词获取身份证号
                    result.put("hanLPIdCard",getHanLIdCard(txtContent));
                    //2、根据相似度身份证号获取参数
                    result.put("isIdCard",isSimilarity(Arrays.asList("身份证号","ID number"),txtContent));
                    //3、根据正则表达式提取身份证号
                    String regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" +
                            "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";
                    result.put("regularIdCard",regularAcquisition(regularExpression,txtContent));

                    /**
                     * 获取生日年龄
                     */
                    //1、性别分词获取年龄
                    result.put("hanLPBirthday",getHanLBirthday(txtContent));
                    //2、根据相似度年龄获取参数
                    result.put("isBirthday",isSimilarity(Arrays.asList("年龄岁"),txtContent));
                    //3、根据正则表达式提取年龄
                    result.put("regularBirthday",regularAcquisition("\\d{1,2}(岁|)",txtContent));

                    /**
                     * 读取学历
                     */
                    //1、分词获取学历
                    result.put("hanLPEducation",getHanLEducation(txtContent));
                    //2、根据相似度学历获取参数
                    result.put("isEducation",isSimilarity(Arrays.asList("小学","初中","高中","职高",
                            "中专","大专","专科","本科","学士","研究生","硕士","博士"),txtContent));
                    //3、根据正则表达式提取学历
                    //result.put("regularEducation",regularAcquisition("",txtContent));

                    /**
                     * 获取学校
                     */
                    //1、分词获取学历
                    result.put("hanLPSchool",getHanLSchool(txtContent));

                    /**
                     * 根据相似度判断工作经验
                     */
                    //获取数词
                    result.put("hanLPWorkExperience",getHanLBirthday(txtContent));
                    result.put("isWorkExperience",isSimilarity(Arrays.asList("工作经验"),txtContent));

                    /**
                     * 获取民族
                     */
                    result.put("hanLPNation",getNation(txtContent));

                    /**
                     * 居住地提取
                     */
                    //居住地

                }
                resultMap.put(txtContent,result);
            }
        }
        return resultMap;
    }

    public static boolean isTitle(String str){

        List<String> list = Arrays.asList("项目经历","掌握技能","技术要点","技能特点",
                "教育经历", "自我评价", "个人信息", "求职意向","工作经历",
                "证书","培训经历","技能技巧","兴趣爱好","面试评价","个人评价",
                "简历","个人简历","教育背景","专业技能","工作经验","项目经验");
        for (String title:list) {
            double titleSimilarity = Similarity.charBasedSimilarity(title, StringFilter(str));
            if(titleSimilarity >= 0.8 )return true;
        }
        return false;
    }

    public static String StringFilter(String str) throws PatternSyntaxException {
        // 只允许中文
        // String regEx="[^a-zA-Z]";
        // 清除掉[]中所有特殊字符
        String regEx = "[`~☆★!@#$%^&*()+=|{}':;,\\[\\]》·.<>/?~！@#￥%……（）——+|{}【】‘；：”“’。，、？a-zA-Z\\s*]";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(str);
        return m.replaceAll("").trim().replace(" ", "").replace("\\", "");
    }

    public static double isFullName(String str){
        List<String> list = Arrays.asList("姓名","名称","名字");
        double fullNameSimilarity = 0.0;
        for (String fullName:list) {
            double similarity = Similarity.charBasedSimilarity(fullName, str);
            if(similarity > fullNameSimilarity){
                fullNameSimilarity = similarity;
            }
        }
        return fullNameSimilarity;
    }


    public static String regularFullName(String str) {
        String result = "";
        String regex = "^(\\s|)姓名(：|:|-|)(\\s|)[\\u4e00-\\u9fa5]{1,6}$";
        Pattern compile = Pattern.compile(regex);
        Matcher matcher = compile.matcher(str);
        if(matcher.find()) {
            result = matcher.group() ;
        }
        return result;
    }

    public static String regularAcquisition(String regex,String str){
        String result = "";
        Pattern compile = Pattern.compile(regex);
        Matcher matcher = compile.matcher(str);
        if(matcher.find()) {
            result = matcher.group() ;
        }
        return result;
    }



    public static String getHanLPSex(String str){
        List<Term> termList = HanLP.segment(str);
        //读取性别
        List<Term> collect = termList.stream().filter(x -> x.nature.toString().equals("性别")).collect(Collectors.toList());
        String sex = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return sex;
    }

    public static String getCompanyName(String str){
        List<Term> companyTerm =  HanLP.segment(str);
        List<Term> collect = companyTerm.stream().filter(x -> x.nature.equals(Nature.nis) && x.word.contains("公司") ).collect(Collectors.toList());
        String companyName = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return companyName;
   }


    public static String getHanLPTelephone(String str){
        List<Term> termList = HanLP.segment(str);
        //读取手机号
        List<Term> collect = termList.stream().filter(x -> x.word.matches("1[3-9]\\d{9}")).collect(Collectors.toList());
        String phone = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return phone;
    }

    public static String getHanLEmail(String str){
        List<Term> termList = HanLP.segment(str);
        List<Term> collect = termList.stream().filter(x -> x.word.matches("^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$")).collect(Collectors.toList());
        String email = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return email;
    }

    public static String getHanLIdCard(String str){
        List<Term> termList = HanLP.segment(str);
        //读取身份证号
        List<Term> collect = termList.stream().filter(x -> isIDNumber(x.word)).collect(Collectors.toList());
        String idCard = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return idCard;
    }

    public static String getHanLBirthday(String str){
        List<Term> termList = HanLP.segment(str);
        List<Term> collect = termList.stream().filter(x -> x.nature.equals(Nature.m)).collect(Collectors.toList());
        String birthday = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return birthday;
    }
    public static String getHanLEducation(String str){
        List<Term> termList = HanLP.segment(str);
        List<Term>  collect = termList.stream().filter(x -> x.nature.toString().equals("学历")).collect(Collectors.toList());
        String education = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return education;
    }


    public static String getHanLSchool(String str){
        List<Term> termList = HanLP.segment(str);
        List<Term> collect = termList.stream().filter(x -> x.nature.equals(Nature.ntu)||x.nature.equals(Nature.nts)).collect(Collectors.toList());
        String school = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return school;
    }

    public static String getNation(String str){
        List<Term> termList = HanLP.segment(str);
        List<Term> collect = termList.stream().filter(x -> x.nature.toString().equals("民族")).collect(Collectors.toList());
        String nation = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
        return nation;
    }


    public static double isSimilarity(List<String> list,String str){
        double fullNameSimilarity = 0.0;
        for (String fullName:list) {
            double similarity = Similarity.charBasedSimilarity(fullName, str);
            if(similarity > fullNameSimilarity){
                fullNameSimilarity = similarity;
            }
        }
        return fullNameSimilarity;
    }

    public static double isMorphoSimilarity(List<String> list,String str){
        double fullNameSimilarity = 0.0;
        for (String fullName:list) {
            double similarity = Similarity.morphoSimilarity(fullName, str);
            if(similarity > fullNameSimilarity){
                fullNameSimilarity = similarity;
            }
        }
        return fullNameSimilarity;
    }



    public static Map<String, Object> analyzeResume(List<String> docContent) {


        for (int c = 0; c < docContent.size(); c++) {

            Map<String, Object> result = new HashMap<String, Object>();

            String  txtContent = docContent.get(c);

            String word1 = "姓名名字名称";

            double conceptSimilarityResult = Similarity.conceptSimilarity(word1, txtContent);
            double charBasedSimilarityResult = Similarity.charBasedSimilarity(word1, txtContent);

//            System.out.println(word1 + " vs " + word2 + " 概念相似度值：" + conceptSimilarityResult);
//            System.out.println(word1 + " vs " + word2 + " 字面相似度值：" + charBasedSimilarityResult);

            //读取人名
            System.out.println(getRelationName(txtContent));

            result.put("姓名",standardParticiple(txtContent).toString());
            //读取人名
//        System.out.println(nlpTokenizer(txtContent));

            List<Term> termList = HanLP.segment(txtContent);

            //读取性别
            List<Term> collect = termList.stream().filter(x -> x.nature.toString().equals("性别")).collect(Collectors.toList());
            String sex = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
            result.put("性别",sex);

            //读取手机号
            collect = termList.stream().filter(x -> x.word.matches("1[3-9]\\d{9}")).collect(Collectors.toList());
            String phone = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
            result.put("手机号",phone);
            //读取邮箱
            collect = termList.stream().filter(x -> x.word.matches("^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$")).collect(Collectors.toList());
            String email = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
            result.put("邮箱",email);
            //读取身份证号
            collect = termList.stream().filter(x -> isIDNumber(x.word)).collect(Collectors.toList());
            String idCard = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
            result.put("身份证号",idCard);
            //读取生日
            //生日
            String[] array = txtContent.split("\\n");
            String birthday = "";
//            Double s = 0.0;
//            for (String str: array) {
//                Double dble = CosineSimilarity.getSimilarity("生日年龄", str);
//                int res =  dble.compareTo(s);
//                if(res > 0) {
//                    s = dble;
//                    birthday = str;
//                }
//            }
            if(StringUtils.isNotEmpty(birthday)){
                termList = HanLP.segment(birthday);
                String sr = "^\\d{4}|(0?[1-9]|1[0-2])|\\d{4}(.|-|\\/|年)\\d{1,2}|\\d{4}(.|-|\\/|年)\\d{1,2}(.|-|\\/|月)\\d{1,2}";
                collect = termList.stream().filter(x -> x.nature.equals(Nature.m) && x.word.matches(sr)).collect(Collectors.toList());
                String strList = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
                result.put("生日",strList);
            }

            //读取学历
            termList = HanLP.segment(txtContent);
            collect = termList.stream().filter(x -> x.nature.toString().equals("学历")).collect(Collectors.toList());
            List<String> strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("学历",strList);

            //读取政治面貌
            collect = termList.stream().filter(x -> x.nature.toString().equals("政治面貌")).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("政治面貌",strList);

            //民族
            collect = termList.stream().filter(x -> x.nature.toString().equals("民族")).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("民族",strList);

            //国籍
            collect = termList.stream().filter(x -> x.nature.toString().equals("国籍")).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("国籍",strList);

            //户口类型
            collect = termList.stream().filter(x -> x.nature.toString().equals("户口类型")).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("户口类型",strList);

            //技能
            collect = termList.stream().filter(x -> x.nature.toString().equals("技能")).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("技能",strList.toString());

            //获取学校
            collect = termList.stream().filter(x -> x.nature.equals(Nature.ntu)||x.nature.equals(Nature.nts)).collect(Collectors.toList());
            strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
            result.put("学校",strList);

            //年工作经验
            String experience = "";
            double s = 0.0;
//            for (String str: array) {
//                Double dble = CosineSimilarity.getSimilarity("工作经验", str);
//                int res =  dble.compareTo(s);
//                if(res > 0) {
//                    s = dble;
//                    experience = str;
//                }
//            }
            if(StringUtils.isNotEmpty(experience)){
                termList = HanLP.segment(experience);
                String sr = "^\\d{1,2}";
                collect = termList.stream().filter(x -> x.nature.equals(Nature.m) && x.word.matches(sr)).collect(Collectors.toList());
                experience = collect.stream().map(term -> term.word).collect(Collectors.toList()).toString();
                result.put("工作经验",experience);
            }

            //婚姻状况
//        collect = termList.stream().filter(x -> x.nature.equals(Nature.c)).collect(Collectors.toList());
//        strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
//        System.out.println("婚姻状况:"+strList);

            /**
             * 个人优势
             */
            String evaluation = "";
            if(txtContent.contains("个人优势")){
                String reult = customStringsplit("(?=个人优势)([\\n\\s\\S]*?)(?=(项目经历|教育经历|自我评价|个人信息|求职意向|工作经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"个人优势");
                reult = reult.substring(4);
                if(StringUtils.isNotEmpty(reult)){
                    evaluation += reult;
                }
            }
            /**
             * 自我评价
             */
            if(txtContent.contains("自我评价")) {
                String reult = customStringsplit("(?=自我评价)([\\n\\s\\S]*?)(?=(个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"自我评价");
                reult = reult.substring(4);
                if(StringUtils.isNotEmpty(reult)){
                    evaluation += reult;
                }
            }
            /**
             * 面试评价
             */
            if(txtContent.contains("面试评价")) {
                String reult = customStringsplit("(?=面试评价)([\\n\\s\\S]*?)(?=(简历|个人简历|个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"面试评价");
                reult = reult.substring(4);
                if(StringUtils.isNotEmpty(reult)){
                    evaluation += reult;
                }
            }
            result.put("自我评价",evaluation);


            /**
             * 工作经历
             */
            if(txtContent.contains("工作经历")) {
                String reult = customStringsplit("(?=工作经历)([\\n\\s\\S]*?)(?=(个人优势|教育经历|自我评价|个人信息|求职意向|项目经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"工作经历");
                reult = reult.substring(4).trim();
//            System.out.println(result);

                /**
                 * 公司定义相似度名称
                 */
                Map<String,String> companyMap = new HashMap<>();
                String[] arr = reult.split("\\n");
                List<String> companyList = new ArrayList<>();
                List<String> dateList = new ArrayList<>();
                for (String str: arr) {
                    Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                    Matcher m = p.matcher(str);
                    str = m.replaceAll("");
                    //读取单位名称
                    List<Term> companyTerm =  HanLP.segment(str);
                    collect = companyTerm.stream().filter(x -> x.nature.equals(Nature.nis) && x.word.contains("公司") ).collect(Collectors.toList());
                    strList = collect.stream().map(term -> term.word).collect(Collectors.toList());
                    if(strList!=null && strList.size()>0){
                        companyList.add(str);
                    }
                    //
                    String SR = "^\\d{4}(.|-|\\/|年)\\d{1,2}(~|-|\\/)(\\d{4}(.|-|\\/|年)\\d{1,2}|至今)";
                    Pattern pattern = Pattern.compile(SR); //尝试提取这样类型的数据
                    Matcher matcher = pattern.matcher(str);
                    while (matcher.find()) {
                        dateList.add(matcher.group());
                    }
                }
                //循环单位名称
                for (int i = 0; i < companyList.size(); i++) {
                    if(i+1<companyList.size()){
                        String ss = customStringsplit( "(?="+companyList.get(i)+")([\\n\\s\\S]*?)(?="+companyList.get(i+1)+")"
                                ,reult,companyList.get(i)).replaceAll(companyList.get(i),"");
                        //.replaceAll(dateList.get(i),"")
                        companyMap.put(companyList.get(i),ss);
                    }else{
                        String ss = customStringsplit( "(?="+companyList.get(i)+")([\\n\\s\\S]*?)?/g"
                                ,reult,companyList.get(i)).replaceAll(companyList.get(i),"");
                        //.replaceAll(dateList.get(i),"")
                        companyMap.put(companyList.get(i),ss);
                    }
                }
                //companyList
                //dateList
                //companyMap

                Map<String,Object> map = new HashMap<>();
                map.put("单位名称",companyList);
                map.put("工作时间",dateList);
                map.put("工作内容",companyMap);
                result.put("工作经历",map);
            }

            /**
             * 项目经历
             */
            if(txtContent.contains("项目经历")) {
                String reult = customStringsplit("(?=项目经历)([\\n\\s\\S]*?)(?=(个人优势|教育经历|自我评价|个人信息|求职意向|工作经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"项目经历");
                reult = reult.substring(4);
                reult = reult.trim();
//            System.out.println(reult);
                List<String> dateList = new ArrayList<>();
                String[] arr = reult.split("\\r\\n");
                for(int i = 0 ; i < arr.length ; i ++){
                    Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                    Matcher m = p.matcher(arr[i]);
                    String str = m.replaceAll("");

                    String SR = "^\\d{4}(.|-|\\/|年)\\d{1,2}(~|-|\\/)(\\d{4}(.|-|\\/|年)\\d{1,2}|至今)";
                    Pattern pattern = Pattern.compile(SR); //尝试提取这样类型的数据
                    Matcher matcher = pattern.matcher(str);
                    while (matcher.find()) {
                        dateList.add(matcher.group());
                    }
                }
                int arrSoluble = 0;
                Map<Integer,Integer> mapSoluble = new HashMap<>();
                for(int i = 0 ; i < arr.length ; i ++){
                    if(null !=dateList && dateList.size()>0){
                        Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                        Matcher m = p.matcher(arr[i]);
                        String str = m.replaceAll("");
                        if(str.equals(dateList.get(0)))arrSoluble = i;
                        for (int j = 0; j < dateList.size(); j++) {
                            if(str.equals(dateList.get(j))){
                                mapSoluble.put(i - arrSoluble,i - arrSoluble);
                            }
                        }
                    }
                }
                List<Map.Entry<Integer,Integer>> lstEntry=new ArrayList<>(mapSoluble.entrySet());
                Collections.sort(lstEntry,((o1, o2) -> {
                    return o1.getValue().compareTo(o2.getValue());
                }));
                List<Integer> list = new ArrayList<>();
                lstEntry.forEach(o -> list.add(o.getValue()));

                List<Map<String,String>> xmjl = new ArrayList<Map<String,String>>();
                if(!list.isEmpty()){
                    for (int i = 0; i < list.size(); i++) {
                        Map<String,String> map = new HashMap<>();
                        if(i+1<list.size()){
                            for (int j = 0; j < arr.length; j++) {
                                if(j>=list.get(i)&&j<list.get(i+1)){
                                    map.put(String.valueOf(j),arr[j]);
                                }
                            }
                        }else{
                            for (int j = 0; j < arr.length; j++) {
                                if(j>=list.get(i)){
                                    map.put(String.valueOf(j),arr[j]);
                                }
                            }
                        }
                        xmjl.add(map);
                    }
                }

//            List<String> companyList = new ArrayList<>();
//            for (int i = 0; i < arr.length; i++) {
//                Pattern p = Pattern.compile("\\s*|\t|\r|\n");
//                Matcher m = p.matcher(arr[i]);
//                String str = m.replaceAll("");
//
//                System.out.println(str);
//
//                for (int j = 0; j < dateList.size(); j++) {
//                    if(arr[i] == dateList.get(j)){
//
//                    }
//
//
//
//                }
//            }
                //companyMap
//            System.out.println(companyList);
//            System.out.println(dateList);

            }

            /**
             * 教育经历
             */
            if(txtContent.contains("教育经历")) {
                String reult = customStringsplit("(?=教育经历)([\\n\\s\\S]*?)(?=(个人优势|项目经历|自我评价|个人信息|求职意向|工作经历|证书|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"教育经历");
                reult = reult.substring(4);
                result.put("教育经历",reult);
            }

            /**
             * 证书
             */
            if(txtContent.contains("证书")) {
                String reult = customStringsplit("(?=证书)([\\n\\s\\S]*?)(?=(个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|自我评价|培训经历|技能技巧|兴趣爱好))"
                        ,txtContent,"证书");
                reult = reult.substring(4);
                result.put("证书",reult);
            }

            /**
             * 培训经历
             */
            if(txtContent.contains("培训经历")) {
                String reult = customStringsplit("(?=培训经历)([\\n\\s\\S]*?)(?=(个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|证书|自我评价|技能技巧|兴趣爱好))"
                        ,txtContent,"培训经历");
                reult = reult.substring(4);
                result.put("培训经历",reult);
            }

            /**
             * 技能技巧
             */
            if(txtContent.contains("技能技巧")) {
                String reult = customStringsplit("(?=技能技巧)([\\n\\s\\S]*?)(?=(个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|证书|自我评价|培训经历|兴趣爱好))"
                        ,txtContent,"技能技巧");
                reult = reult.substring(4);
                result.put("技能技巧",reult);
            }

            /**
             * 兴趣爱好
             */
            if(txtContent.contains("兴趣爱好")) {
                String reult = customStringsplit("(?=兴趣爱好)([\\n\\s\\S]*?)(?=(个人优势|项目经历|教育经历|个人信息|求职意向|工作经历|证书|自我评价|培训经历|技能技巧))"
                        ,txtContent,"兴趣爱好");
                reult = reult.substring(4);
                result.put("技能技巧",reult);
            }

            System.out.println(result);
        }

        return null;
    }

    //定义方法max，实现求某个double数组的最大值，并返回
    public static double max(double arr[]){
        double max = arr[0];
        for ( int i=0;i<arr.length ;i++ ) {
            if (max<arr[i]) {
                max=arr[i];
            }
        }
        return max;
    }

    /**
     *
     * @param regular 截取正则表达式
     * @param str 截取字符串
     * @param splitName 如果字符串包含且为最末尾拆分字符串使用
     * @return
     */
    public static String customStringsplit(String regular ,String str,String splitName){
        String result = "";
        Pattern compile = Pattern.compile(regular, Pattern.CASE_INSENSITIVE);
        Matcher matcher = compile.matcher(str);
        while (matcher.find()) {
            int i = 1;
            if (StringUtils.isNotEmpty(matcher.group(i))) {
                result = matcher.group(i);
            }
            i++;
        }
        if(StringUtils.isEmpty(result)&&StringUtils.isNotEmpty(str)&&str.split(splitName).length>1){
            result = str.split(splitName)[str.split(splitName).length-1];
        }
        return result;
    }

    public static Map<String, String> sortMapByKey(Map<String, String> map) {
        if (map == null || map.isEmpty()) {
            return null;
        }

        Map<String, String> sortMap = new TreeMap<String, String>(new Comparator<String>() {
            @Override
            public int compare(String o1,String o2) {

                return ((String)o1).compareTo((String) o2);
            }
        });

        sortMap.putAll(map);

        return sortMap;
    }

}
