package com.ruoyi.web.controller.dingding;

import com.aliyun.dingtalkcontact_1_0.models.GetUserHeaders;
import com.aliyun.dingtalkcontact_1_0.models.GetUserResponse;
import com.aliyun.dingtalkcontact_1_0.Client;
import com.aliyun.dingtalkoauth2_1_0.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.*;
import com.dingtalk.api.response.*;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.dingding.config.DingAppConfig;
import com.ruoyi.common.utils.dingding.config.DingUrlConstant;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.resume.CompanyInfo;
import com.ruoyi.system.domain.resume.CompanyUser;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.resume.ICompanyInfoService;
import com.ruoyi.system.service.resume.ICompanyUserService;
import com.ruoyi.web.controller.dingding.service.DingAuthServiceInfo;
import com.ruoyi.web.controller.dingding.service.DingTokenService;
import com.taobao.api.ApiException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>DingLoginController 此类用于：钉钉企业内部应用免登（H5微应用）</p>
 * <p>@remark：钉钉企业内部微应用DEMO, 实现了身份验证（免登）功能</p>
 */
@Api(value = "dingAuthController", tags = "钉钉企业内部应用免登（H5微应用）")
@Controller
@RequestMapping(value = "/ding")
@Slf4j
@CrossOrigin
public class DingAuthController {
    @Resource
    private DingAppConfig dingAppConfig;

    @Resource
    private DingTokenService dingTokenService;

    @Resource
    private DingAuthServiceInfo dingAuthServiceInfo;

    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ICompanyInfoService companyInfoService;

    @Autowired
    private ICompanyUserService companyUserService;
    @Resource
    private RedisCache redisCache;







    /**
     * 访问/user/getuserinfo接口获取用户userId
     *
     * @param accessToken access_token
     * @param authCode    临时授权码
     * @return 用户userId或错误信息
     */
    private AjaxResult getUserInfo(String accessToken, String authCode) {
        DingTalkClient client = new DefaultDingTalkClient(DingUrlConstant.URL_GET_USER_INFO);
        OapiUserGetuserinfoRequest request = new OapiUserGetuserinfoRequest();
        request.setCode(authCode);
        request.setHttpMethod("GET");
        OapiUserGetuserinfoResponse response;
        try {
            response = client.execute(request, accessToken);
        } catch (ApiException e) {
            return AjaxResult.error(e.getErrCode(), "Failed to getUserInfo: " + e.getErrMsg());
        }
        if (!response.isSuccess()) {
            return AjaxResult.error(response.getErrorCode(), response.getErrmsg());
        }
        return AjaxResult.success(response.getUserid());
    }

    /**
     * 访问/user/get 获取用户名称
     *
     * @param accessToken access_token
     * @param userId      用户userId
     * @return 用户名称或错误信息
     */
    private OapiV2UserGetResponse getAuthUser(String accessToken, String userId) {
        DingTalkClient client = new DefaultDingTalkClient(DingUrlConstant.URL_USER_GET_V2);
        OapiV2UserGetResponse response;
        OapiV2UserGetRequest request = new OapiV2UserGetRequest();
        request.setUserid(userId);
        request.setLanguage("en_US");
        try {
            response = client.execute(request, accessToken);
        } catch (ApiException e) {
            log.error("Failed to getUserName: " + e.getErrMsg());
            return null;
        }
        return response;
    }

    /**
     * 访问/user/get 获取用户名称
     *
     * @param accessToken access_token
     * @param userId      用户userId
     * @return 用户名称或错误信息
     */
    private AjaxResult getUser(String accessToken, String userId) {
        DingTalkClient client = new DefaultDingTalkClient(DingUrlConstant.URL_USER_GET);
        OapiUserGetRequest request = new OapiUserGetRequest();
        request.setUserid(userId);
        request.setHttpMethod("GET");

        OapiUserGetResponse response;
        try {
            response = client.execute(request, accessToken);
        } catch (ApiException e) {
            return AjaxResult.error(Integer.parseInt(e.getErrCode()), "Failed to getUserName: " + e.getErrMsg());
        }

        SysUser user = this.assembleUserDTO(response, accessToken);

        return AjaxResult.success(user);
    }

    /**
     * 封装返回的用户信息
     *
     * @param response    response
     * @param accessToken accessToken
     * @return 用户信息
     */
    private SysUser assembleUserDTO(OapiUserGetResponse response, String accessToken) {
        String userid = response.getUserid();
        String mobile = response.getMobile();
        try {
            SysUser sysUser = Optional.ofNullable(dingAuthServiceInfo.getUserByUserId(userid))
                    .orElse(dingAuthServiceInfo.getUserByUsername(mobile));
            sysUser.setAccessToken(accessToken);
            sysUser.setNickName(response.getName());
            sysUser.setAvatar(response.getAvatar());
            return sysUser;
        } catch (Exception e) {
            return new SysUser();
        }
    }


    @RequestMapping(value = "/getAdminInfo", method = RequestMethod.GET)
    public AjaxResult getAdminInfo(@RequestParam("authCode") String authCode) {
        try {
            com.aliyun.dingtalkoauth2_1_0.Client client = this.createClientNew();
            String accessToken = dingTokenService.getAdminUserAccessToken();
            GetSsoUserInfoHeaders getSsoUserInfoHeaders = new GetSsoUserInfoHeaders();
            getSsoUserInfoHeaders.xAcsDingtalkAccessToken = accessToken;
            GetSsoUserInfoRequest getSsoUserInfoRequest = new GetSsoUserInfoRequest()
                    .setCode(authCode);
            // 获取应用管理员的身份信息
            GetSsoUserInfoResponse response = client.getSsoUserInfoWithOptions(getSsoUserInfoRequest, getSsoUserInfoHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        return AjaxResult.success();
    }

    /**
     * 根据保存的code值和获取的access_token，获取应用管理身份信息
     * openapi@dingtalk
     *
     * @param code
     * @param access_token
     * @return OapiSsoGetuserinfoResponse
     * 2020-11-3
     */
    public OapiSsoGetuserinfoResponse getAdminUserInfo(String code, String access_token) {
        DingTalkClient client = new DefaultDingTalkClient(DingUrlConstant.URL_SSO_USER_INFO);
        OapiSsoGetuserinfoRequest request = new OapiSsoGetuserinfoRequest();
        request.setCode(code);
        request.setHttpMethod("GET");
        OapiSsoGetuserinfoResponse response;
        try {
            response = client.execute(request, access_token);
        } catch (ApiException e) {
            e.printStackTrace();
            return null;
        }
        return response;
    }


    /**
     * 钉钉用户登录
     *
     * @param authCode 免登临时authCode
     * @return 当前用户
     */
    @ApiOperation(value = "钉钉内部应用-用户登录")
    @PostMapping("/authToken2")
    @ResponseBody
    public AjaxResult authToken2(@RequestBody String authCode) {
        // 获取accessToken
        try {
            AjaxResult accessTokenSr = dingTokenService.getUserAccessToken(authCode);

            Client client = this.createClient();
            GetUserHeaders getUserHeaders = new GetUserHeaders();
            getUserHeaders.xAcsDingtalkAccessToken = accessTokenSr.get(accessTokenSr.MSG_TAG).toString();
            try {
                GetUserResponse response = client.getUserWithOptions("me", getUserHeaders, new RuntimeOptions());
                accessTokenSr.put("sysUser", response.getBody());

                SysUser sysUser = dingAuthServiceInfo.getUserByUsername(response.getBody().getMobile());

                if (StringUtils.isNull(sysUser)) {
                    // todo 生成用户信息进行登录
                }

                String token = loginService.authSingleSignIn(sysUser.getUserName());
                if (StringUtils.isNotEmpty(token)) {
                    accessTokenSr.put(Constants.TOKEN, token);
                }
            } catch (TeaException e) {
                return AjaxResult.error(e.getMessage(), e);
            } catch (Exception e) {
                return AjaxResult.error(e.getMessage(), e);
            }
            return accessTokenSr;
        } catch (Exception e) {
            return AjaxResult.error("获取用户登录请求异常!");
        }
    }


    /**
     * 钉钉登录不存在用户自动增加用户以及组织信息
     *
     * @param response
     * @return
     */
    private SysUser addUserAndOrg(OapiV2UserGetResponse response) {
        SysUser newUser = new SysUser();
        newUser.setUserName(response.getResult().getMobile());
        newUser.setAvatar(response.getResult().getAvatar());
        newUser.setNickName(response.getResult().getName());
        newUser.setEmail(response.getResult().getEmail());
        newUser.setCreateBy("admin");
        newUser.setPassword(SecurityUtils.encryptPassword("admin123"));
        //角色自动赋值为简历管理角色
        Long[] roleIds = new Long[1];
        SysRole role = roleService.selectRoleByKey("resume");
        if (role != null) {
            roleIds[0] = role.getRoleId();
        }
        newUser.setRoleIds(roleIds);
        newUser.setPhonenumber(newUser.getUserName());
        userService.insertUser(newUser);
        //关联企业信息
        String corpId = response.getResult().getUnionEmpExt().getCorpId();
        CompanyInfo companyInfo = new CompanyInfo();
        companyInfo.setCorpId(corpId);
        List<CompanyInfo> companyInfoList = companyInfoService.selectCompanyInfoList(companyInfo);
        if (companyInfoList != null && !companyInfoList.isEmpty()) {
            CompanyUser companyUser = new CompanyUser();
            companyUser.setCompanyId(companyInfoList.get(0).getId());
            companyUser.setUserId(newUser.getUserId());
            companyUserService.insertCompanyUser(companyUser);
        }
        return newUser;
    }

    /**
     * 使用 Token 初始化账号Client
     *
     * @return Client
     * @throws Exception
     */
    public static Client createClient() throws Exception {
        Config config = new Config();
        config.protocol = "https";
        config.regionId = "central";
        return new Client(config);
    }

    /**
     * 使用 Token 初始化账号后台Client
     *
     * @return Client
     * @throws Exception
     */
    private static com.aliyun.dingtalkoauth2_1_0.Client createClientNew() throws Exception {
        Config config = new Config();
        config.protocol = "https";
        config.regionId = "central";
        return new com.aliyun.dingtalkoauth2_1_0.Client(config);
    }


}