package com.ruoyi.web.controller.dingding;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkcontact_1_0.models.GetUserResponse;
import com.aliyun.dingtalkoauth2_1_0.models.GetUserTokenResponse;
import com.dingtalk.api.response.*;
import com.fasterxml.jackson.databind.JsonNode;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.web.service.PermissionService;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.impl.SysRoleServiceImpl;
import com.ruoyi.tool.domain.AuthApply;
import com.ruoyi.tool.service.IAuthApplyService;
import com.ruoyi.web.controller.dingding.request.login.AuthLoginRequest;
import com.ruoyi.web.controller.dingding.service.DingAuthServiceInfo;
import com.ruoyi.web.controller.dingding.service.DingThirdTokenService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>DingLoginController 此类用于：钉钉企业内部应用免登（H5微应用）</p>
 * <p>@remark：钉钉企业内部微应用DEMO, 实现了身份验证（免登）功能</p>
 */
@Api(value = "dingAuthController", tags = "钉钉第三方企业应用免登（H5微应用）")
@Controller
@RequestMapping(value = "/auth/ding")
@Slf4j
public class DingThirdAuthController extends BaseController {

    @Resource
    private DingThirdTokenService dingAuthTokenService;

    @Resource
    private DingAuthServiceInfo dingAuthServiceInfo;

    @Autowired
    private SysLoginService loginService;
    @Autowired
    private PermissionService permissionService;
    @Autowired
    private IAuthApplyService authApplyService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ISysUserService sysUserService;
    @Resource
    private SysUserMapper userMapper;
    @Resource
    private SysRoleMapper sysRoleMapper;
    @Autowired
    private SysUserRoleMapper userRoleMapper;


    @PostMapping(value = "/callback")
    @ResponseBody
    public Map<String, String> callback(@RequestParam String signature,
                                        @RequestParam String timestamp,
                                        @RequestParam String nonce,
                                        @RequestBody JsonNode encryptNode) {
        String encryptMsg = encryptNode.get("encrypt").textValue();
        String plainText = dingAuthServiceInfo.decryptText(signature, timestamp, nonce, encryptMsg);
        JSONObject callBackContent = JSON.parseObject(plainText);

        //进入回调事件分支选择
        Map<String, String> resultMap = dingAuthServiceInfo.caseProcess(callBackContent);
        return resultMap;
    }

//
//    @ApiOperation(value = "第三方企业应用用户登录")
//    @PostMapping("/authLogin")
//    @ResponseBody
//    public AjaxResult authLogin(@RequestBody AuthLoginRequest authLoginRequest) {
//        log.info("钉钉用户登录第三方企业应用code:{},corpId:{}", authLoginRequest.getCode(), authLoginRequest.getCorpId());
//        //调用接口获取第三方企业应用的access_token，详情请参考获取第三方应用授权企业的accessToken。
////        String corpAccessToken = dingAuthTokenService.getCorpAccessToken(authLoginRequest.getCorpId());
//        GetUserTokenResponse userTokenResponse = dingAuthTokenService.getUserAccessToken(authLoginRequest.getCode());
//        log.info("用户登录信息corpAccessToken" + JSONObject.toJSONString(userTokenResponse));
//        String corpAccessToken = userTokenResponse.getBody().getAccessToken();
//        //获取用户userid。
////        OapiV2UserGetuserinfoResponse userUnfo = dingAuthTokenService.getUserUnfo(authLoginRequest.getCode(), corpAccessToken);
////        log.info("用户登录信息" + JSONObject.toJSONString(userUnfo));
////        //调用接口获取用户的userid，详情请参考通过免登码获取用户信息。
////        OapiV2UserGetResponse authUser = dingAuthTokenService.getAuthUser(corpAccessToken, userUnfo.getResult().getUserid());
////
////        log.info("用户登录详细信息" + JSONObject.toJSONString(authUser.getResult()));
//
//        GetUserResponse addressBookUserInfo = dingAuthTokenService.getAddressBookUserInfo(corpAccessToken, "me");
//        log.info("用户通讯录信息" + JSONObject.toJSONString(addressBookUserInfo));
//        //获取用户详情。
//        SysUser sysUser = dingAuthServiceInfo.getUserByUserId(addressBookUserInfo.getBody().getMobile());
//        if (StringUtils.isNull(sysUser)) {
//            // todo 生成用户信息进行登录
//            dingAuthServiceInfo.initUser(addressBookUserInfo.getBody(), authLoginRequest.getCorpId());
//        }
//        String token = loginService.authDingThirdLogin(addressBookUserInfo.getBody().getMobile(), authLoginRequest.getCorpId());
//        return AjaxResult.success(token);
//    }

    @ApiOperation(value = "第三方企业应用授权")
    @PostMapping("/cropLogin")
    @ResponseBody
    public AjaxResult cropLogin(@RequestBody AuthLoginRequest authLoginRequest) {

        log.info("钉钉用户登录第三方企业应用code:{},corpId:{}", authLoginRequest.getCode(), authLoginRequest.getCorpId());
        //调用接口获取第三方企业应用的access_token，详情请参考获取第三方应用授权企业的accessToken。
        String corpAccessToken = dingAuthTokenService.getCorpAccessToken(authLoginRequest.getCorpId());
//        GetUserTokenResponse userTokenResponse = dingAuthTokenService.getUserAccessToken(authLoginRequest.getCode());
        log.info("用户登录信息corpAccessToken" + corpAccessToken);
        //获取用户userid。
        OapiV2UserGetuserinfoResponse userUnfo = dingAuthTokenService.getUserUnfo(authLoginRequest.getCode(), corpAccessToken);
        log.info("用户登录信息" + JSONObject.toJSONString(userUnfo));
        //调用接口获取用户的userid，详情请参考通过免登码获取用户信息。
        OapiV2UserGetResponse authUser = dingAuthTokenService.getAuthUser(corpAccessToken, userUnfo.getResult().getUserid());

        log.info("用户登录详细信息" + JSONObject.toJSONString(authUser.getResult()));

//        GetUserResponse addressBookUserInfo = dingAuthTokenService.getAddressBookUserInfo(corpAccessToken, "me");
//        log.info("用户通讯录信息" + JSONObject.toJSONString(addressBookUserInfo));
        //获取用户详情。
//        SysUser sysUser = dingAuthServiceInfo.getUserByUserNameAndCorpId(userUnfo.getResult().getUserid(), authLoginRequest.getCorpId());
//        if (StringUtils.isNull(sysUser)) {
        // todo 初始化用户
        dingAuthServiceInfo.initUser(authUser.getResult(), authLoginRequest.getCorpId());
//        }
        String token = loginService.authDingThirdLogin(authUser.getResult().getUserid(), authLoginRequest.getCorpId());
        return AjaxResult.success(token);
    }

    @ApiOperation(value = "用户向企业管理员提交授权申请")
    @PostMapping("/cropAuth/apply")
    @ResponseBody
    public AjaxResult cropAuthApply() {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roles = "tentant_admin,hr";
        boolean hasAnyRoles = permissionService.hasAnyRoles(roles);
        if (hasAnyRoles) {
            AjaxResult.error("您已具备该企业权限，无需再次申请");
        }
        authApplyService.applyAuth(loginUser.getUser());
        return AjaxResult.success("申请成功，请等待管理员审核");
    }

    @ApiOperation(value = "用户向企业管理员提交授权申请列表")
    @GetMapping("/cropAuth/apply/page")
    @ResponseBody
    @ApiResponses(
            @ApiResponse(code = 200, message = "成功", response = AuthApply.class)
    )
    public TableDataInfo cropAuthApplyPage() {
        startPage();
        LoginUser loginUser = SecurityUtils.getLoginUser();
        List<AuthApply> list = authApplyService.applyAuthList(0, loginUser.getUser().getCorpId());
        return getDataTable(list);
    }

    @ApiOperation(value = "管理员审批用户授权申请")
    @PostMapping("/cropAuth/complete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "申请id", paramType = "query", dataType = "Long", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "flag", value = "审批", paramType = "query", dataType = "Boolean", dataTypeClass = Boolean.class),
    })
    @ResponseBody
    public AjaxResult cropAuthComplete(Long id, Boolean flag) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        authApplyService.applyComplete(id, flag, loginUser.getUserId());
        return AjaxResult.success();
    }

    @ApiOperation(value = "管理员给用户开通企业权限")
    @PostMapping("/cropAuth/liberal")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "empIds", value = "员工ids", paramType = "query",dataType = "Collection", dataTypeClass = Collection.class),
    })
    @ResponseBody
    public AjaxResult cropAuthLiberal(String[] empIds) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysRole role = sysRoleService.selectRoleByKey("hr");
        Long[] roleIds = new Long[1];
        roleIds[0] = role.getRoleId();
        for (String empId : empIds) {
            SysUser sysUser = userMapper.selectByEmployNoAndCorpId(empId, loginUser.getCorpid());
            if (sysUser == null) {
                //查询不到用户证明 该员工还未登录，先由管理员赋予hr角色
                sysUser = new SysUser();
                sysUser.setEmployNo(empId);
                sysUser.setCorpId(loginUser.getCorpid());
                sysUser.setCreateBy(loginUser.getUser().getUserId().toString());
                sysUser.setCreateTime(new Date());
                userMapper.insertUser(sysUser);

                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(sysUser.getUserId());
                userRole.setRoleId(role.getRoleId());
                sysUserService.insertUserAuth(sysUser.getUserId(), roleIds);
            } else {
                //有用户信息 设置他的角色为hr
                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(sysUser.getUserId());
                userRole.setRoleId(role.getRoleId());
                sysUserService.insertUserAuth(sysUser.getUserId(), roleIds);
            }
        }
        return AjaxResult.success();
    }
}