package com.ruoyi.web.controller.dingding.service;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.aliyun.dingtalkcontact_1_0.models.GetUserResponseBody;
import com.dingtalk.api.response.OapiV2DepartmentGetResponse;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.dingtalk.oapi.lib.aes.DingTalkEncryptException;
import com.dingtalk.oapi.lib.aes.DingTalkEncryptor;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.resume.CompanyInfo;
import com.ruoyi.system.domain.resume.CompanyUser;
import com.ruoyi.system.mapper.SysMenuMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.resume.ICompanyInfoService;
import com.ruoyi.system.service.resume.ICompanyUserService;
import com.ruoyi.tool.domain.TCompanyUser;
import com.ruoyi.tool.domain.TEnterpriseCompany;
import com.ruoyi.tool.service.ITCompanyUserService;
import com.ruoyi.tool.service.ITEnterpriseCompanyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.controller.dingding.config.SyncHttpConstant;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.ruoyi.web.controller.dingding.config.CallbackConstant.*;

/**
 * <p>DingAuthServiceImpl 此类用于：</p>
 * <p>@author：hujm</p>
 * <p>@date：2021年05月20日 15:56</p>
 * <p>@remark：</p>
 */
@Slf4j
@Service
public class DingAuthServiceInfoImpl implements DingAuthServiceInfo {

    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ICompanyInfoService companyInfoService;

    @Autowired
    private ICompanyUserService companyUserService;
    @Resource
    private RedisCache redisCache;
    @Resource
    private SysMenuMapper sysMenuMapper;
    @Resource
    private SysRoleMapper sysRoleMapper;

    @Autowired
    private ITEnterpriseCompanyService tEnterpriseCompanyService;

    @Autowired
    private ITCompanyUserService tCompanyUserService;
    @Autowired
    private DingDeptService dingDeptService;
    @Autowired
    private DingThirdTokenService dingThirdTokenService;
    @Autowired
    private ISysRoleService sysRoleService;

    @Override
    public SysUser getUserByUserId(String userId) {

        if (userId == null) {
            log.error("根据用户手机号查询UIOT用户时，当前钉钉用户id为空！");
            return new SysUser();
        }

        SysUser sysUser = sysUserMapper.selectUserByUserName(userId);

        return sysUser;
    }


    @Override
    public SysUser getUserByUsername(String mobile) {

        if (mobile == null) {
            log.error("根据用户手机号查询UIOT用户时，当前钉钉用户id为空！");
            return new SysUser();
        }

        SysUser sysUser = sysUserMapper.selectUserByUserName(mobile);

        return sysUser;
    }

    @Override
    public SysUser addUserAndOrg(OapiV2UserGetResponse response) {
        SysUser newUser = new SysUser();
        newUser.setUserName(response.getResult().getUserid());
        newUser.setAvatar(response.getResult().getAvatar());
        newUser.setNickName(response.getResult().getName());
        newUser.setEmail(response.getResult().getEmail());
        newUser.setCreateBy("admin");
        newUser.setPassword(SecurityUtils.encryptPassword("admin123"));
        //角色自动赋值为简历管理角色
        Long[] roleIds = new Long[1];
        SysRole role = roleService.selectRoleByKey("resume");
        if (role != null) {
            roleIds[0] = role.getRoleId();
        }
        List<Long> menuIds = sysMenuMapper.selectMenuListByRoleId(role.getRoleId(), role.isMenuCheckStrictly());
        newUser.setMenuIds(menuIds.toArray(new Long[0]));
        newUser.setRoleIds(roleIds);
        newUser.setPhonenumber(newUser.getUserName());
        userService.insertUser(newUser);
        //关联企业信息
        String corpId = response.getResult().getUnionEmpExt().getCorpId();
        CompanyInfo companyInfo = new CompanyInfo();
        companyInfo.setCorpId(corpId);
        List<CompanyInfo> companyInfoList = companyInfoService.selectCompanyInfoList(companyInfo);
        if (companyInfoList != null && !companyInfoList.isEmpty()) {
            CompanyUser companyUser = new CompanyUser();
            companyUser.setCompanyId(companyInfoList.get(0).getId());
            companyUser.setUserId(newUser.getUserId());
            companyUserService.insertCompanyUser(companyUser);
        }
        return newUser;
    }

    @Override
    public SysUser initUser(OapiV2UserGetResponse.UserGetResponse response, String corpId) {
        String userid = response.getUserid();
        String jobNumber = response.getJobNumber();
        SysUser sysUser = userService.selectByEmployNoAndCorpId(jobNumber, corpId);
        sysUser.setUserName(userid);
        sysUser.setAvatar(response.getAvatar());
        sysUser.setNickName(response.getName());
        sysUser.setEmail(response.getEmail());
        if (sysUser != null){
            sysUserMapper.updateUser(sysUser);
        }else {
            sysUser.setCorpId(corpId);
            sysUser.setCreateBy("1");
            sysUserMapper.insertUser(sysUser);
            //初始化角色  为游客
            SysRole role = sysRoleService.selectRoleByKey("tourist");
            Long[] roleIds = new Long[1];
            roleIds[0] = role.getRoleId();
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(sysUser.getUserId());
            userRole.setRoleId(role.getRoleId());
            userService.insertUserAuth(sysUser.getUserId(), roleIds);
        }
        return sysUser;
    }

    @Override
    public String decryptText(String signature, String timestamp, String nonce, String encryptMsg) {
        String plainText = "";
        try {
            DingTalkEncryptor dingTalkEncryptor = new DingTalkEncryptor(SyncHttpConstant.TOKEN, SyncHttpConstant.ENCODING_AES_KEY, SyncHttpConstant.SUITE_KEY);
            plainText = dingTalkEncryptor.getDecryptMsg(signature, timestamp, nonce, encryptMsg);
        } catch (DingTalkEncryptException e) {
            log.error("钉钉消息体解密错误, signature: {}, timestamp: {}, nonce: {}, encryptMsg: {}, e: {}", signature, timestamp, nonce, encryptMsg, e);
        }
        log.debug("钉钉消息体解密, signature: {}, timestamp: {}, nonce: {}, encryptMsg: {}, 解密结果: {}", signature, timestamp, nonce, encryptMsg, plainText);
        return plainText;
    }
    @Override
    public Map<String, String> caseProcess(JSONObject plainNode) {
        Map<String, String> resultMap = new LinkedHashMap<>();
        String eventType = plainNode.getString("EventType");
        JSONArray bizData = plainNode.getJSONArray("bizData");
        String corp_id = null;
        JSONObject biz_data = null;
        Integer bizType = 0;


        switch (eventType) {
            case SUITE_TICKET_CALLBACK_URL_VALIDATE:
                log.info("[callback] 验证回调地址有效性质:{}", plainNode);
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case TEMP_AUTH_CODE_ACTIVE:
                log.info("[callback] 企业开通授权:{}", plainNode);
                log.info("[callback] 企业开通授权信息：", bizData);
                if (bizType == 4 && biz_data != null && corp_id != null) {
                    updateCompanyInfo(biz_data,corp_id);
                }
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case SUITE_RELIEVE:
                log.info("[callback] 企业解除授权:{}", plainNode);
                if (bizType == 4 && biz_data != null && corp_id != null) {
                    tEnterpriseCompanyService.updateTCompanyUserClose(corp_id);
                }
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case CHECK_UPDATE_SUITE_URL:
                log.info("[callback] 在开发者后台修改回调地址:" + plainNode);
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case CHECK_CREATE_SUITE_URL:
                log.info("[callback] 检查钉钉向回调URL POST数据解密后是否成功:" + plainNode);
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case CONTACT_CHANGE_AUTH:
                log.info("[callback] 通讯录授权范围变更事件:" + plainNode);
                break;
            case ORG_MICRO_APP_STOP:
                log.info("[callback] 停用应用:" + plainNode);
                break;
            case ORG_MICRO_APP_RESTORE:
                log.info("[callback] 启用应用:" + plainNode);
                break;
            case MARKET_BUY:
                log.info("[callback] 用户下单购买事件:" + plainNode);
                break;
            case SYNC_HTTP_PUSH_HIGH:
                log.info("[callback] 套件票据事件:" + plainNode);
                for (Object bizDatum : bizData) {
                    JSONObject bizDataJson = (JSONObject) bizDatum;
                    log.info("[callback] 获取到授权bizDataJson:" + bizDataJson);
                    String biz_data_str = bizDataJson.getString("biz_data");
                    biz_data = JSONObject.parseObject(biz_data_str);
                    bizType = bizDataJson.getInteger("biz_type");
                    log.info("[callback] 获取到bizType:" + bizType);
                    corp_id = bizDataJson.getString("corp_id");
                }
                log.info("[callback] 获取到套件票据corp_id:{}", corp_id);
                //套件票据
                if (bizType == 2 && biz_data != null && corp_id != null) {
                    String suiteTicket = biz_data.getString("suiteTicket");
                    String redisKeyPrefix = Constants.DAILY_DING_AUTH + corp_id + ":";
                    redisCache.setCacheObject(redisKeyPrefix + Constants.DAILY_DING_SUITE_TICKET, suiteTicket, 5, TimeUnit.HOURS);
                }
                //套件票据授权
                if (bizType == 4 && biz_data != null && corp_id != null) {
                    //syncAction字段的取值如下
                    //org_suite_auth：表示企业授权第三方企业应用
                    //org_suite_change：表示企业变更授权范围
                    //org_suite_relieve：表示企业解除授权
                    //企业授权信息
                    String syncAction = biz_data.getString("syncAction");
                    if ("org_suite_auth".equals(syncAction)) {
                        log.info("[callback] 企业授权第三方企业应用:{}",corp_id);
                        updateCompanyInfo(biz_data,corp_id);
                    } else if ("org_suite_change".equals(syncAction)) {
                        log.info("[callback] 表示企业变更授权范围:{}",corp_id);
                    } else if ("org_suite_relieve".equals(syncAction)) {
                        log.info("[callback] 表示企业解除授权:{}",corp_id);
                    }
                }
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            case CHECK_URL:
                log.info("[callback] 检查钉钉向回调URL POST数据解密后是否成功:" + plainNode);
                resultMap = encryptText(CALLBACK_RETURN_SUCCESS);
                break;
            default:
                log.info("[callback] 未知事件: {} , 内容: {}", eventType, plainNode);
                resultMap = encryptText("事件类型未定义, 请联系应用提供方!" + eventType);
                break;
        }
        return resultMap;
    }
    @Override
    public Map<String, String> encryptText(String text) {
        Map<String, String> resultMap = new LinkedHashMap<>();
        try {
            DingTalkEncryptor dingTalkEncryptor = new DingTalkEncryptor(SyncHttpConstant.TOKEN, SyncHttpConstant.ENCODING_AES_KEY, SyncHttpConstant.SUITE_KEY);
            resultMap = dingTalkEncryptor.getEncryptedMap(text, System.currentTimeMillis(), getRandomStr(8));
        } catch (DingTalkEncryptException e) {
            log.error("钉钉消息体加密,text: {}, e: {}", text, e);
        }
        log.debug("钉钉消息体加密,text: {}, resultMap: {}", text, resultMap);
        return resultMap;
    }
    public static String getRandomStr(int count) {
        String base = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < count; ++i) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }

        return sb.toString();
    }

    public void updateCompanyInfo(JSONObject biz_data, String corp_id) {
        String authCorpInfo_str = biz_data.getString("auth_corp_info");//授权方企业信息。
        String auth_info_str = biz_data.getString("auth_info");//授权信息。
        String auth_user_info_str = biz_data.getString("auth_user_info");//授权方管理员信息
        JSONObject authCorpInfo = JSONObject.parseObject(authCorpInfo_str);
        JSONObject authInfo = JSONObject.parseObject(auth_info_str);
        JSONArray agentArray = authInfo.getJSONArray("agent");
        JSONObject agent = JSONObject.parseObject(agentArray.get(0).toString());
        JSONObject authUserInfo = JSONObject.parseObject(auth_user_info_str);
//        String accessToken = dingThirdTokenService.getCorpAccessToken(corp_id);
        String manager_userid = authUserInfo.getString("userId");
        TCompanyUser po = new TCompanyUser();
        po.setUserId(manager_userid);
        po.setCorpid(corp_id);
        //企业管理员
        List<TCompanyUser> poList = tCompanyUserService.selectTCompanyUserList(po);
        //用户详细信息
//        OapiV2UserGetResponse authUser = dingThirdTokenService.getAuthUser(accessToken, manager_userid);
//        OapiV2UserGetResponse.UserGetResponse result = authUser.getResult();
        if (CollUtil.isEmpty(poList)) {
            tCompanyUserService.insertTCompanyUser(po);
            // 插入用户
            SysUser user = new SysUser();
//            user.setDeptIdList(result.getDeptIdList());
//            user.setNickName(result.getName());
            user.setUserName(manager_userid);
//            user.setEmail(result.getEmail());
//            user.setPhonenumber(result.getMobile());
//            user.setAvatar(result.getAvatar());
            user.setStatus("0");
//            user.setEmployNo(result.getJobNumber());
            user.setCorpId(corp_id);
            user.setCreateBy("1");
            sysUserMapper.insertUser(user);
            // 插入用户角色 赋予租户管理员
            SysRole role = sysRoleService.selectRoleByKey("tentant_admin");
            Long[] roleIds = new Long[1];
            roleIds[0] = role.getRoleId();
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(user.getUserId());
            userRole.setRoleId(role.getRoleId());
            userService.insertUserAuth(user.getUserId(), roleIds);
        }
        // 查询企业信息
        TEnterpriseCompany tEnterpriseCompany = tEnterpriseCompanyService.selectTEnterpriseCompanyByCorpId(authCorpInfo.getString("corpid"));

        if (null != tEnterpriseCompany) {
            tEnterpriseCompany.setName(authCorpInfo.getString("corp_name"));
            tEnterpriseCompany.setLogoUrl(authCorpInfo.getString("corp_logo_url"));
            tEnterpriseCompany.setCorpid(authCorpInfo.getString("corpid"));
            tEnterpriseCompany.setInviteCode(authCorpInfo.getString("invite_code"));
            tEnterpriseCompany.setIndustry(authCorpInfo.getString("industry"));
            tEnterpriseCompany.setCorpName(authCorpInfo.getString("corp_name"));
            tEnterpriseCompany.setLicenseCode(authCorpInfo.getString("license_code"));
            tEnterpriseCompany.setAuthChannel(authCorpInfo.getString("auth_channel"));
            tEnterpriseCompany.setAuthChannelType(authCorpInfo.getString("auth_channel_type"));
            tEnterpriseCompany.setIsAuthenticated(authCorpInfo.get("is_authenticated").toString());
            tEnterpriseCompany.setAuthLevel(authUserInfo.getString("auth_level"));
            tEnterpriseCompany.setInviteUrl(authUserInfo.getString("invite_url"));
            tEnterpriseCompany.setCorpLogoUrl(authUserInfo.getString("corp_logo_url"));
            tEnterpriseCompany.setAgentid(agent.getString("agentid"));
            tEnterpriseCompany.setName(agent.getString("agent_name"));
            tEnterpriseCompany.setLogoUrl(agent.getString("logo_url"));
            tEnterpriseCompanyService.updateTEnterpriseCompany(tEnterpriseCompany);
        } else {
            tEnterpriseCompany = new TEnterpriseCompany();
            tEnterpriseCompany.setName(authCorpInfo.getString("corp_name"));
            tEnterpriseCompany.setLogoUrl(authCorpInfo.getString("corp_logo_url"));
            tEnterpriseCompany.setCorpid(authCorpInfo.getString("corpid"));
            tEnterpriseCompany.setInviteCode(authCorpInfo.getString("invite_code"));
            tEnterpriseCompany.setIndustry(authCorpInfo.getString("industry"));
            tEnterpriseCompany.setCorpName(authCorpInfo.getString("corp_name"));
            tEnterpriseCompany.setLicenseCode(authCorpInfo.getString("license_code"));
            tEnterpriseCompany.setAuthChannel(authCorpInfo.getString("auth_channel"));
            tEnterpriseCompany.setAuthChannelType(authCorpInfo.getString("auth_channel_type"));
            tEnterpriseCompany.setIsAuthenticated(authCorpInfo.get("is_authenticated").toString());
            tEnterpriseCompany.setAuthLevel(authUserInfo.getString("auth_level"));
            tEnterpriseCompany.setInviteUrl(authUserInfo.getString("invite_url"));
            tEnterpriseCompany.setCorpLogoUrl(authUserInfo.getString("corp_logo_url"));
            tEnterpriseCompany.setAgentid(agent.getString("agentid"));
            tEnterpriseCompany.setName(agent.getString("agent_name"));
            tEnterpriseCompany.setLogoUrl(agent.getString("logo_url"));
            tEnterpriseCompanyService.insertTEnterpriseCompany(tEnterpriseCompany);
        }

    }
    /**
     * 更新企业详细信息
     * @param biz_data
     * @param corp_id
     */
    public void updateCompanyInfo2(JSONObject biz_data, String corp_id){
        String authCorpInfo_str = biz_data.getString("auth_corp_info");
        JSONObject authCorpInfo = JSONObject.parseObject(authCorpInfo_str);
        String authInfo_str = biz_data.getString("auth_info");
        JSONObject authInfo = JSONObject.parseObject(authInfo_str);
        JSONArray agent = authInfo.getJSONArray("agent");
        JSONObject agent_json = JSONObject.parseObject(agent.get(0).toString());
        JSONArray admin_list = agent_json.getJSONArray("admin_list");
        List<String> adminList = admin_list.stream().map(data -> data.toString()).collect(Collectors.toList());
        String agentid = agent_json.getString("agentid");
        String agentName = agent_json.getString("agent_name");
        String logoUrl = agent_json.getString("logo_url");
        String accessToken = dingThirdTokenService.getCorpAccessToken(corp_id);
        // 输出结果
        String finalCorp_id = corp_id;
        TCompanyUser po = new TCompanyUser();
        po.setAgentid(agentid);
        po.setCorpid(finalCorp_id);
        //企业管理员
        List<TCompanyUser> poList = tCompanyUserService.selectTCompanyUserList(po);
        adminList.forEach(data -> {
            po.setUserId(data);
            boolean exists = poList.stream()
                    .anyMatch(companyUser -> companyUser.getUserId().equals(data)); // 检查userId是否已存在
            if (!exists) {
                tCompanyUserService.insertTCompanyUser(po);
                OapiV2UserGetResponse authUser = dingThirdTokenService.getAuthUser(accessToken, data);
                OapiV2UserGetResponse.UserGetResponse result = authUser.getResult();
                // 插入用户
                SysUser user = new SysUser();
                user.setDeptIdList(result.getDeptIdList());
                user.setNickName(result.getName());
                user.setUserName(data);
                user.setEmail(result.getEmail());
                user.setPhonenumber(result.getMobile());
                user.setAvatar(result.getAvatar());
                user.setStatus("0");
                user.setEmployNo(result.getJobNumber());
                user.setCorpId(corp_id);
                sysUserMapper.insertUser(user);
                // 插入用户角色 赋予租户管理员

                SysRole role = roleService.selectRoleByKey("resume");
                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(user.getUserId());
                userRole.setRoleId(role.getRoleId());
                sysRoleMapper.insertRole(role);
            }
        });

        // 查询企业信息
        TEnterpriseCompany tEnterpriseCompany = tEnterpriseCompanyService.selectTEnterpriseCompanyByCorpId(authCorpInfo.getString("corpid"));
        if(null != tEnterpriseCompany){
            TEnterpriseCompany other = JSON.toJavaObject(authCorpInfo, TEnterpriseCompany.class);
            other.setAgentid(agentid);
            other.setName(agentName);
            other.setLogoUrl(logoUrl);
            tEnterpriseCompany.copyFrom(other,true);
            tEnterpriseCompanyService.updateTEnterpriseCompany(tEnterpriseCompany);
        } else {
            // 转换JSONObject为TEnterpriseCompany对象
            tEnterpriseCompany = JSON.toJavaObject(authCorpInfo, TEnterpriseCompany.class);
            tEnterpriseCompany.setAgentid(agentid);
            tEnterpriseCompany.setName(agentName);
            tEnterpriseCompany.setLogoUrl(logoUrl);
            tEnterpriseCompanyService.insertTEnterpriseCompany(tEnterpriseCompany);
        }

    }

}