package com.ruoyi.web.controller.moonshot;


import cn.moonshot.platform.util.MoonshotAiUtils;
import com.google.gson.Gson;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.FileType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.moonshot.Client;
import com.ruoyi.common.utils.moonshot.vo.ChatCompletionRequest;
import com.ruoyi.common.utils.moonshot.vo.ChatCompletionStreamChoice;
import com.ruoyi.common.utils.moonshot.vo.FileUploadResult;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.system.domain.resume.Resume;
import com.ruoyi.system.service.resume.IResumeService;
import com.ruoyi.tool.domain.TUnifyFile;
import com.ruoyi.tool.service.ITUnifyFileService;
import com.ruoyi.web.controller.common.CommonController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping("/moonshot/file")
public class MoonshotFileController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    @Autowired
    private ITUnifyFileService tUnifyFileService;

    @Value("${moonshot.api_key}")
    private String API_KEY;

    @Autowired
    private IResumeService resumeService;

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception
    {
        AjaxResult ajax = AjaxResult.success();
        String corpId = "";
        if (StringUtils.isNotEmpty(SecurityUtils.getLoginUser().getCorpid())) {
            corpId = SecurityUtils.getLoginUser().getCorpid();
        }
        try
        {
            if (API_KEY == null) {
                System.out.println("Please set MOONSHOT_API_KEY env");
                ajax.put("API_KEY","接口API_KEY不存在");
                return ajax;
            }
            Client client = new Client(API_KEY);

            FileUploadResult resuat = client.uploadFile(file);

            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;
            TUnifyFile tUnifyFile = new TUnifyFile();
            tUnifyFile.setId(IdUtils.fastSimpleUUID());
            tUnifyFile.setUploadPath(url);
            tUnifyFile.setUploadName(fileName);
            tUnifyFile.setNewUploadName(FileUtils.getName(fileName));
            tUnifyFile.setUploadFormat(file.getOriginalFilename());
            tUnifyFile.setUploadType(String.valueOf(FileType.RESUME.ordinal()));//简历附件
            tUnifyFile.setCreateBy(getNickName());
            tUnifyFile.setCorpId(corpId);

            //moonshot AI 附件上传返回附件信息
            if(null != resuat.getError()){
                tUnifyFile.setStatus_details(resuat.getError().getMessage());
            }else if(resuat.getStatus().equals("ok")) {
                tUnifyFile.copyFrom(resuat,true);
                /**
                 * 根据moonshot获取附件解析内容
                 */
                if(StringUtils.isNotEmpty(tUnifyFile.getFileId())){
                    tUnifyFile.setContent(MoonshotAiUtils.getFileContent(tUnifyFile.getFileId(),client.getApiKey()));
                    //读取内容后删除AI存储附件
                    MoonshotAiUtils.deleteFile(tUnifyFile.getFileId(),client.getApiKey());
                }
            }

            // todo 开始简历解析
            if(StringUtils.isNotEmpty(tUnifyFile.getContent())){

                StringBuilder str = new StringBuilder();

                client.ChatCompletionStream(new ChatCompletionRequest(tUnifyFile.getContent())).subscribe(
                        streamResponse -> {
                            if (streamResponse.getChoices().isEmpty()) {
                                return;
                            }
                            for (ChatCompletionStreamChoice choice : streamResponse.getChoices()) {
                                String finishReason = choice.getFinishReason();
                                if (finishReason != null) {
                                    continue;
                                }
                                // 将每个choice的delta内容添加到StringBuilder中
                                str.append(choice.getDelta().getContent());
                            }
                        },
                        error -> {
                            error.printStackTrace();
                        },
                        () -> {
                            System.out.println(str);
                            // todo 格式解析完成-存储简历内容整理并返回
                            if(isValidJson((str.toString()))){
                                Gson gson = new Gson();
                                Resume resume = gson.fromJson(str.toString(), Resume.class); // 使用 Gson 转换 JSON 字符串为 Resume 对象
                                resumeService.insertResume(resume);
                                tUnifyFile.setFileBusinessId(resume.getResumeId().toString());
                            }else{
                                // 返回值不是json字符串
                                ajax.put("error","解析失败，AI解析返回格式有误，请重试！");
                            }
                        }
                );
            }

            tUnifyFileService.insertTUnifyFile(tUnifyFile);

            //根据上传附件已经moonshot返回上传内容
            ajax.put(tUnifyFile.getFileId(),tUnifyFile);
            return ajax;
        }
        catch (Exception e)
        {
            return AjaxResult.error(e.getMessage());
        }
    }



    public static boolean isValidJson(String test) {
        try {
            new Gson().fromJson(test, Resume.class);
            return true;
        } catch (com.google.gson.JsonSyntaxException ex) {
            // 如果捕获到JsonSyntaxException异常，则字符串不是有效的JSON
            return false;
        }
    }

}
