package com.ruoyi.web.controller.tool;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.tool.domain.TCompanyUser;
import com.ruoyi.tool.service.ITCompanyUserService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 企业管理员Controller
 * 
 * @author ruoyi
 * @date 2024-06-04
 */
@RestController
@RequestMapping("/tool/authUser")
public class TCompanyUserController extends BaseController
{
    @Autowired
    private ITCompanyUserService tCompanyUserService;

    /**
     * 查询企业管理员列表
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:list')")
    @GetMapping("/list")
    public TableDataInfo list(TCompanyUser tCompanyUser)
    {
        startPage();
        List<TCompanyUser> list = tCompanyUserService.selectTCompanyUserList(tCompanyUser);
        return getDataTable(list);
    }

    /**
     * 导出企业管理员列表
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:export')")
    @Log(title = "企业管理员", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TCompanyUser tCompanyUser)
    {
        List<TCompanyUser> list = tCompanyUserService.selectTCompanyUserList(tCompanyUser);
        ExcelUtil<TCompanyUser> util = new ExcelUtil<TCompanyUser>(TCompanyUser.class);
        util.exportExcel(response, list, "企业管理员数据");
    }

    /**
     * 获取企业管理员详细信息
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return success(tCompanyUserService.selectTCompanyUserById(id));
    }

    /**
     * 新增企业管理员
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:add')")
    @Log(title = "企业管理员", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TCompanyUser tCompanyUser)
    {
        return toAjax(tCompanyUserService.insertTCompanyUser(tCompanyUser));
    }

    /**
     * 修改企业管理员
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:edit')")
    @Log(title = "企业管理员", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TCompanyUser tCompanyUser)
    {
        return toAjax(tCompanyUserService.updateTCompanyUser(tCompanyUser));
    }

    /**
     * 删除企业管理员
     */
    @PreAuthorize("@ss.hasPermi('tool:authUser:remove')")
    @Log(title = "企业管理员", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(tCompanyUserService.deleteTCompanyUserByIds(ids));
    }
}
