package com.ruoyi.web.controller.tool;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.tool.domain.TEnterpriseCompany;
import com.ruoyi.tool.service.ITEnterpriseCompanyService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 企业信息管理Controller
 * 
 * @author ruoyi
 * @date 2024-06-03
 */
@RestController
@RequestMapping("/tool/company")
public class TEnterpriseCompanyController extends BaseController
{
    @Autowired
    private ITEnterpriseCompanyService tEnterpriseCompanyService;

    /**
     * 查询企业信息管理列表
     */
    @PreAuthorize("@ss.hasPermi('tool:company:list')")
    @GetMapping("/list")
    public TableDataInfo list(TEnterpriseCompany tEnterpriseCompany)
    {
        startPage();


        List<TEnterpriseCompany> list = tEnterpriseCompanyService.selectTEnterpriseCompanyList(tEnterpriseCompany);
        return getDataTable(list);
    }

    /**
     * 导出企业信息管理列表
     */
    @PreAuthorize("@ss.hasPermi('tool:company:export')")
    @Log(title = "企业信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TEnterpriseCompany tEnterpriseCompany)
    {
        List<TEnterpriseCompany> list = tEnterpriseCompanyService.selectTEnterpriseCompanyList(tEnterpriseCompany);
        ExcelUtil<TEnterpriseCompany> util = new ExcelUtil<TEnterpriseCompany>(TEnterpriseCompany.class);
        util.exportExcel(response, list, "企业信息管理数据");
    }

    /**
     * 获取企业信息管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('tool:company:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return success(tEnterpriseCompanyService.selectTEnterpriseCompanyById(id));
    }

    /**
     * 新增企业信息管理
     */
    @PreAuthorize("@ss.hasPermi('tool:company:add')")
    @Log(title = "企业信息管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TEnterpriseCompany tEnterpriseCompany)
    {
        return toAjax(tEnterpriseCompanyService.insertTEnterpriseCompany(tEnterpriseCompany));
    }

    /**
     * 修改企业信息管理
     */
    @PreAuthorize("@ss.hasPermi('tool:company:edit')")
    @Log(title = "企业信息管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TEnterpriseCompany tEnterpriseCompany)
    {
        return toAjax(tEnterpriseCompanyService.updateTEnterpriseCompany(tEnterpriseCompany));
    }

    /**
     * 删除企业信息管理
     */
    @PreAuthorize("@ss.hasPermi('tool:company:remove')")
    @Log(title = "企业信息管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(tEnterpriseCompanyService.deleteTEnterpriseCompanyByIds(ids));
    }
}
