package com.ruoyi.web.controller.tool.mobile;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.system.domain.resume.Resume;
import com.ruoyi.system.domain.resume.ResumeFile;
import com.ruoyi.system.service.resume.IResumeService;
import com.ruoyi.utils.resumeAnalysis.FileUtils;
import com.ruoyi.utils.resumeAnalysis.ResumeAnalysisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 简历管理Controller
 * 
 * @author ruoyi
 * @date 2024-02-18
 */
@RestController
@RequestMapping("/mobile/resume")
public class ResumeMobileController extends BaseController
{
    @Autowired
    private IResumeService resumeService;

    @Autowired
    private ServerConfig serverConfig;

    /**
     * 查询简历管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(Resume resume)
    {
        startPage();
        List<Resume> list = resumeService.selectResumeList(resume);
        return getDataTable(list);
    }

    /**
     * 获取简历管理详细信息
     */
    @GetMapping(value = "/{resumeId}")
    public AjaxResult getInfo(@PathVariable("resumeId") Long resumeId)
    {

        return success(resumeService.selectResumeByResumeId(resumeId,SecurityUtils.getLoginUser().getPrincipal()));
    }

    /**
     * 新增简历管理
     */
    @PostMapping
    public AjaxResult add(@RequestBody Resume resume)
    {
        return toAjax(resumeService.insertResume(resume));
    }

    /**
     * 方法描述 上传简历并进行解析
     * @author
     * @date
     */
    @PostMapping(value = "/uploadResumeSimilarity",consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ResponseBody
    public AjaxResult uploadResumeSimilarity(@RequestParam(value = "file") MultipartFile file) {
        try{
            if (file == null || StringUtils.isBlank(file.getOriginalFilename())) {
                return AjaxResult.error("请上传简历");
            }
            String path = RuoYiConfig.getUploadPath();
            String originalFilename = file.getOriginalFilename();
            String suffix = originalFilename.substring(originalFilename.lastIndexOf("."));
            String fileName = FileUploadUtils.upload(path, file);

            // 上传位置
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
            String ymd = sdf.format(new Date());
            String lastPath = "/" + ymd + "/";
            String fileUrl = path + lastPath + fileName;

            if(suffix.equals(".doc")||suffix.equals(".docx")){
                List<String> content = null;
                try {
                    if(suffix.equals(".doc")){
                        content = FileUtils.readWord(fileUrl);
                    } else if (suffix.equals(".docx")) {
                        content = FileUtils.readWordDocx(fileUrl);
                    }
                } catch (Exception e) {
                    return AjaxResult.error("文件不存在解析失败");
                }
                //处理标识分段，并判断内容情况
                Resume resume = ResumeAnalysisUtil.toResumeAnalysis(content);
                //处理文件信息
                String url = serverConfig.getUrl() + fileName;
                String newFileName = com.ruoyi.common.utils.file.FileUtils.getName(fileName);
                ResumeFile resumeFile = new ResumeFile();
                resumeFile.setUploadPath(url);
                resumeFile.setUploadName(newFileName);
                resumeFile.setUploadType(newFileName.split("\\.")[1]);
                resume.setResumeFile(resumeFile);
                return AjaxResult.success(resume);
            }else{
                return AjaxResult.error("文件格式不正确请上传.doc或者.docx格式文件！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error("解析失败！请手动录入");
        }
    }

    /**
     * 修改简历管理
     */
    @PutMapping
    public AjaxResult edit(@RequestBody Resume resume)
    {
        return toAjax(resumeService.updateResume(resume));
    }

    /**
     * 删除简历管理
     */
	@DeleteMapping("/{resumeIds}")
    public AjaxResult remove(@PathVariable Long[] resumeIds)
    {
        return toAjax(resumeService.deleteResumeByResumeIds(resumeIds));
    }
}
