package com.ruoyi.tool.service.impl;

import java.util.List;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.tool.mapper.TEnterpriseCompanyMapper;
import com.ruoyi.tool.domain.TEnterpriseCompany;
import com.ruoyi.tool.service.ITEnterpriseCompanyService;

/**
 * 企业信息管理Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-06-03
 */
@Service
public class TEnterpriseCompanyServiceImpl implements ITEnterpriseCompanyService 
{
    @Autowired
    private TEnterpriseCompanyMapper tEnterpriseCompanyMapper;

    /**
     * 查询企业信息管理
     * 
     * @param id 企业信息管理主键
     * @return 企业信息管理
     */
    @Override
    public TEnterpriseCompany selectTEnterpriseCompanyById(String id)
    {
        return tEnterpriseCompanyMapper.selectTEnterpriseCompanyById(id);
    }

    /**
     * 查询企业信息管理列表
     * 
     * @param tEnterpriseCompany 企业信息管理
     * @return 企业信息管理
     */
    @Override
    @DataScope(mainTable = "a")
    public List<TEnterpriseCompany> selectTEnterpriseCompanyList(TEnterpriseCompany tEnterpriseCompany)
    {
        return tEnterpriseCompanyMapper.selectTEnterpriseCompanyList(tEnterpriseCompany);
    }

    /**
     * 新增企业信息管理
     * 
     * @param tEnterpriseCompany 企业信息管理
     * @return 结果
     */
    @Override
    public int insertTEnterpriseCompany(TEnterpriseCompany tEnterpriseCompany)
    {
        tEnterpriseCompany.setId(IdUtils.fastSimpleUUID());
        tEnterpriseCompany.setCreateTime(DateUtils.getNowDate());
        return tEnterpriseCompanyMapper.insertTEnterpriseCompany(tEnterpriseCompany);
    }

    /**
     * 修改企业信息管理
     * 
     * @param tEnterpriseCompany 企业信息管理
     * @return 结果
     */
    @Override
    public int updateTEnterpriseCompany(TEnterpriseCompany tEnterpriseCompany)
    {
        tEnterpriseCompany.setUpdateTime(DateUtils.getNowDate());
        return tEnterpriseCompanyMapper.updateTEnterpriseCompany(tEnterpriseCompany);
    }

    /**
     * 批量删除企业信息管理
     * 
     * @param ids 需要删除的企业信息管理主键
     * @return 结果
     */
    @Override
    public int deleteTEnterpriseCompanyByIds(String[] ids)
    {
        return tEnterpriseCompanyMapper.deleteTEnterpriseCompanyByIds(ids);
    }

    /**
     * 删除企业信息管理信息
     * 
     * @param id 企业信息管理主键
     * @return 结果
     */
    @Override
    public int deleteTEnterpriseCompanyById(String id)
    {
        return tEnterpriseCompanyMapper.deleteTEnterpriseCompanyById(id);
    }

    /**
     * 根据企业ID查询数据
     * @param corpid
     * @return
     */
    @Override
    public TEnterpriseCompany selectTEnterpriseCompanyByCorpId(String corpid) {
        return tEnterpriseCompanyMapper.selectTEnterpriseCompanyByCorpId(corpid);
    }

    /**
     * 解除授权企业
     * @param corpId
     */
    @Override
    public int updateTCompanyUserClose(String corpId) {
        return tEnterpriseCompanyMapper.updateTCompanyUserClose(corpId);
    }

}
